@@ -7,12 +7,14 @@ sub EXT_controller : Extend('Ext.app.ViewController') : Type('controller') {
         roles => [],
 
         api => {
-            signin    => undef,    # $api->{'Auth/signin'},
-            signout   => undef,    # $api->{'Auth/signout'},
-            setLocale => undef,    # $api->{'Auth/set_locale'},
+            signin          => undef,    # $api->{'Auth/signin'},
+            signout         => undef,    # $api->{'Auth/signout'},
+            setLocale       => undef,    # $api->{'Auth/set_locale'},
+            changePassword  => undef,    # $api->{'Auth/change_password'},
+            recoverPassword => undef,    # $api->{'Auth/recover_password'},
         },
 
-        # FASHION THEME
+        # MATERIAL THEME
         baseColor   => undef,
         accentColor => undef,
         darkMode    => \0,
@@ -24,14 +26,17 @@ sub EXT_controller : Extend('Ext.app.ViewController') : Type('controller') {
 
         listen => {
             global => {
-                mask           => 'mask',
-                unmask         => 'unmask',
-                unmatchedRoute => 'onUnmatchedRoute',
-                remoteEvent    => 'onRemoteEvent',
-                requestError   => 'onRequestError',
-                signin         => 'onSignin',
-                signout        => 'onSignout',
-                setLocale      => 'onSetLocale',
+                mask            => 'mask',
+                unmask          => 'unmask',
+                unmatchedRoute  => 'onUnmatchedRoute',
+                remoteEvent     => 'onRemoteEvent',
+                requestError    => 'onRequestError',
+                signin          => 'onSignin',
+                signout         => 'onSignout',
+                setLocale       => 'onSetLocale',
+                setTheme        => 'setTheme',
+                changePassword  => 'onChangePassword',
+                recoverPassword => 'onRecoverPassword',
             },
         },
 
@@ -39,17 +44,16 @@ sub EXT_controller : Extend('Ext.app.ViewController') : Type('controller') {
             var me = this;
 
             // parse API methods
-            if (me.api.signin) me.api.signin = Ext.direct.Manager.parseMethod(me.api.signin);
-            if (me.api.signout) me.api.signout = Ext.direct.Manager.parseMethod(me.api.signout);
-            if (me.api.setLocale) me.api.setLocale = Ext.direct.Manager.parseMethod(me.api.setLocale);
+            for (var method in me.api) {
+                me.api[method] = Ext.direct.Manager.parseMethod(me.api[method]);
+            }
 
-            if (!!window.Fashion) {
-                var theme = {"dark-mode": me.darkMode ? "true" : "false"};
-                if (me.baseColor) theme["base-color-name"] = me.baseColor;
-                if (me.accentColor) theme["accent-color-name"] = me.accentColor;
-
-                Fashion.css.setVariables(theme);
-            };
+            // set material theme
+            me.setTheme(localStorage.theme != null ? JSON.parse(localStorage.theme) : {
+                darkMode: me.darkMode,
+                base: me.baseColor,
+                accent: me.accentColor
+            });
 
             Ext.util.History.hashbang = true;
 
@@ -119,9 +123,26 @@ JS
 
             if (!session.locale) session.locale = localStorage.locale || session.default_locale;
 
+            session.theme = JSON.parse(localStorage.theme);
+
             return session;
 JS
 
+        # MATERIAL THEME
+        setTheme => func ['theme'], <<"JS",
+            if (Ext.theme.Material) {
+                Ext.manifest.material = Ext.manifest.material || {};
+                Ext.manifest.material.toolbar = Ext.manifest.material.toolbar || {};
+                Ext.manifest.material.toolbar.dynamic = true;
+
+                if (localStorage.theme != null) theme = Ext.apply(JSON.parse(localStorage.theme), theme);
+
+                localStorage.theme = JSON.stringify(theme);
+
+                Ext.theme.Material.setColors(theme);
+            }
+JS
+
         # MASK
         mask => func [], <<'JS',
             this.getView().setMasked(this.defaultMask);
@@ -224,8 +245,8 @@ JS
             });
 JS
 
-        # SIGNIN
-        onSignin => func [ 'username', 'password', 'persistent' ], <<"JS",
+        # USER ACTIONS
+        onSignin => func [ 'username', 'password', 'persistent', 'cb' ], <<"JS",
             var me = this;
 
             me.mask();
@@ -258,9 +279,13 @@ JS
 
                         me.redirectTo(Ext.util.History.getToken(), {force: true});
                     });
+
+                    if (cb) cb(true);
                 }
                 else {
                     Ext.fireEvent('requestError', res);
+
+                    if (cb) cb(false);
                 }
             });
 JS
@@ -283,6 +308,40 @@ JS
             this.redirectTo(Ext.util.History.getToken(), {force: true});
 JS
 
+        onChangePassword => func [ 'password', 'token', 'cb' ], <<"JS",
+            var me = this;
+
+            me.doChangePassword(password, token, function (res) {
+                if (res.isSuccess()) {
+                    Ext.toast($l10n->{'Password changed'}, 5000);
+
+                    if (cb) cb(true);
+                }
+                else {
+                    me.onRequestError(res);
+
+                    if (cb) cb(false);
+                }
+            });
+JS
+
+        onRecoverPassword => func [ 'username', 'cb' ], <<"JS",
+            var me = this;
+
+            me.doRecoverPassword(username, function (res) {
+                if (res.isSuccess()) {
+                    Ext.toast($l10n->{'Email with password change instructions was sent to the email address, associated with your user name.'}, 5000);
+
+                    if (cb) cb(true);
+                }
+                else {
+                    me.onRequestError(res);
+
+                    if (cb) cb(false);
+                }
+            });
+JS
+
         # API
         doSignin => func [ 'username', 'password', 'cb' ], <<"JS",
             this.api.signin({
@@ -298,6 +357,17 @@ JS
         doSetLocale => func ['locale'], <<"JS",
             this.api.setLocale(locale);
 JS
+
+        doChangePassword => func [ 'password', 'token', 'cb' ], <<"JS",
+            this.api.changePassword({
+                password: password,
+                token: token
+            }, cb);
+JS
+
+        doRecoverPassword => func [ 'username', 'cb' ], <<"JS",
+            this.api.recoverPassword(username, cb);
+JS
     };
 }
 
